import json
from typing import List


def load_answers_file(path: str) -> List[str]:
    with open(path) as f:
        answers = [json.loads(line)['answer'] for line in f.readlines()]
    return answers


def load_questions_file(path: str) -> List[str]:
    with open(path) as f:
        questions = [json.loads(line)['question'] for line in f.readlines()]
    return questions



import json

from mini_judge.judge_execution import TrialOutcome


class TrialResultManager:
    """
    Responsible for managing the results of a trial.
    Keeps track of the number of wins for the candidate and reference answers.
    Has serialization methods for the judge answers and trial stats.
    """
    def __init__(self):
        self.candidate_wins = 0
        self.ref_wins = 0
        self.total = 0

        self.judge_answers = []

    def update(self, trial_outcome: TrialOutcome):
        verdict = trial_outcome.trial_verdict
        if verdict == 1:
            self.candidate_wins += 1
        elif verdict == -1:
            self.ref_wins += 1
        self.total += 1
        self.judge_answers.append(trial_outcome.judge_answer)

    def calc_stats(self):
        return {'candidate_win_rate': self.candidate_wins / (self.candidate_wins + self.ref_wins),
                'candidate_wins': self.candidate_wins,
                'ref_wins': self.ref_wins,
                'undecided': self.total - self.candidate_wins - self.ref_wins,
                'total': self.candidate_wins + self.ref_wins}

    def __repr__(self):
        stats = self.calc_stats()

        return f'Candidate Win Rate: {stats["candidate_win_rate"]}\n' \
               f'Candidate Wins: {stats["candidate_wins"]}\n' \
               f'Ref Wins: {stats["ref_wins"]}\n' \
               f'Undecided: {stats["undecided"]}\n' \
               f'Total: {stats["total"]}\n'

    def serialize_judge_answers(self, output_path: str):
        with open(output_path, 'w') as f:
            for answer in self.judge_answers:
                f.write(json.dumps({'judge_answer': f'{answer}'}) + '\n')

    def serialize_stats(self, output_path: str):
        with open(output_path, 'w') as f:
            f.write(json.dumps(self.calc_stats()) + '\n')



import os
from concurrent.futures import ThreadPoolExecutor
from itertools import repeat

import fire
import openai
from dotenv import load_dotenv
from loguru import logger
from mini_judge.data_io import load_answers_file
from mini_judge.data_io import load_questions_file
from mini_judge.judge_execution import trial
from mini_judge.trial_result_manager import TrialResultManager
from pkg_resources import resource_filename
from tqdm import tqdm


def main(judge_model: str = 'gpt-4',
         questions_path: str = resource_filename('mini_judge', 'example_data/questions.jsonl'),
         candidate_answers_path: str = resource_filename('mini_judge', 'example_data/candidate_answers.jsonl'),
         ref_answers_path: str = resource_filename('mini_judge', 'example_data/ref_answers.jsonl'),
         output_dir: str = 'output',
         query_parallelism: int = 2):
    load_dotenv()
    openai.api_key = os.getenv("OPENAI_API_KEY")

    logger.info('Loading data')
    questions = load_questions_file(questions_path)
    candidate_answers = load_answers_file(candidate_answers_path)
    ref_answers = load_answers_file(ref_answers_path)

    assert len(questions) == len(candidate_answers) == len(ref_answers)

    trial_result_mgr = TrialResultManager()
    if query_parallelism > 1:
        with ThreadPoolExecutor(query_parallelism) as executor:
            map_op = executor.map(trial, repeat(judge_model), questions, candidate_answers,
                                  ref_answers)
            for trial_outcome in tqdm(map_op, total=len(candidate_answers), desc='Evaluating'):
                trial_result_mgr.update(trial_outcome)
    else:
        for q, m_a, r_a in tqdm(zip(questions, candidate_answers, ref_answers), desc='Evaluating'):
            trial_outcome = trial(judge_model, q, m_a, r_a)
            trial_result_mgr.update(trial_outcome)

    logger.info(trial_result_mgr)

    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    trial_result_mgr.serialize_judge_answers(os.path.join(output_dir, 'judge_answers.jsonl'))
    trial_result_mgr.serialize_stats(os.path.join(output_dir, 'trial_stats.jsonl'))


if __name__ == '__main__':
    fire.Fire(main)



judge_prompt_template = """
Determine if the candidate answer is better than the reference answer.
If the model answer is better, end your response with a [[YES]].
Otherwise, end your response with a [[NO]].

Question:
{question}

Candidate Answer:
{candidate_answer}

Reference Answer:
{ref_answer}
"""



import time
from dataclasses import dataclass

import openai
from loguru import logger
from mini_judge.judge_prompt import judge_prompt_template

MAX_TOKENS = 2048
API_ERROR_OUTPUT = "API_ERROR"
API_MAX_RETRY = 3
API_RETRY_SLEEP = 10


@dataclass
class TrialOutcome:
    judge_answer: str
    trial_verdict: int


def parse_judge_answer(judge_answer):
    if judge_answer.endswith('[[NO]]'):
        return -1
    elif judge_answer.endswith('[[YES]]'):
        return 1
    return 0


def trial(judge_model: str, question: str, candidate_answer: str, ref_answer: str) -> TrialOutcome:
    prompt = judge_prompt_template.format(question=question, candidate_answer=candidate_answer,
                                          ref_answer=ref_answer)
    messages = format_openai_chat_prompt(prompt)
    judge_answer = chat_completion_openai(judge_model, messages,
                                          temperature=0, max_tokens=MAX_TOKENS)

    trial_verdict = parse_judge_answer(judge_answer)
    return TrialOutcome(judge_answer, trial_verdict)


def format_openai_chat_prompt(prompt_text: str):
    return [{'role': 'user', 'content': prompt_text}]


def chat_completion_openai(model, messages, temperature, max_tokens):
    output = API_ERROR_OUTPUT
    for _ in range(API_MAX_RETRY):
        try:
            response = openai.ChatCompletion.create(
                model=model,
                messages=messages,
                n=1,
                temperature=temperature,
                max_tokens=max_tokens,
            )
            output = response["choices"][0]["message"]["content"]
            break
        except openai.error.OpenAIError as e:
            logger.error(e)
            time.sleep(API_RETRY_SLEEP)

    return output
